
#include "IO_Selector.h"
/*
   (c)2014-2017 Forward Computing and Control Pty. Ltd.
   NSW Australia, www.forward.com.au
   This code is not warranted to be fit for any purpose. You may only use it at your own risk.
   This generated code may be freely used for both private and commercial use
   provided this copyright is maintained.
*/

IO_Selector::IO_Selector(bool _hasPWM, pfodDwgs *_dwgsPtr) : pfodControl(_dwgsPtr) {
  z_idx = pfodDwgs::reserveIdx(4);
  baseCol = 0; baseRow = 0; baseScaling = 1.0f;
  value = 0; // initially input
  setBase(20, 5);
  has_PWM = _hasPWM;
  forceUpdate = false;
  saveCurrentState();
}

void IO_Selector::setBase(int c, int r) {
  baseCol = c;
  baseRow = r;
}

void IO_Selector::setBaseScaling(double _scaling) {
  baseScaling = (float)_scaling;
}

void IO_Selector::forceFullUpdate() {
  forceUpdate = true;
}


char IO_Selector::getCmd_Input() {
  return buttonCmd_Input;
}

char IO_Selector::getCmd_InputPullup() {
  return buttonCmd_InputPullup;
}

char IO_Selector::getCmd_Output() {
  return buttonCmd_Output;
}

char IO_Selector::getCmd_PWM() {
  return buttonCmd_PWM;
}

void IO_Selector::setValue(int newValue) {
  int limit = has_PWM ? 3 : 2;
  if (newValue < 0) {
    newValue = 0;
  } else if (newValue > limit) {
    newValue = limit;
  }
  pfodControl::setValue(newValue);
}


void IO_Selector::draw() {

  dwgsPtr->pushZero(baseCol, baseRow, baseScaling);
  if (label != NULL) { // only send label if one set
    dwgsPtr->label().bold().fontSize(+3).text(label).color(dwgsPtr->BLACK).offset(-18, 0).send();
  }
  dwgsPtr->pushZero(-10, 0, 0.5);
  dwgsPtr->label().text(F("<")).fontSize(+8).offset(1.5, -8).send();
  dwgsPtr->label().text(F("Input")).fontSize(+6).offset(0, 9).send();
  dwgsPtr->circle().filled().color(dwgsPtr->BLACK).radius(3).send();
  dwgsPtr->circle().filled().idx(z_idx).color(dwgsPtr->WHITE).radius(2).send();
  dwgsPtr->popZero();

  dwgsPtr->pushZero(0, 0, 0.5);
  dwgsPtr->label().text(F("<<-3>\u2191")).fontSize(+8).offset(-1, -8).send();
  dwgsPtr->label().text(F("Input\nPullup")).fontSize(+6).offset(0, 9).send();
  dwgsPtr->circle().filled().color(dwgsPtr->BLACK).radius(3).send();
  dwgsPtr->circle().filled().idx(z_idx + 1).color(dwgsPtr->WHITE).radius(2).send();
  dwgsPtr->popZero();

  dwgsPtr->pushZero(10, 0, 0.5);
  dwgsPtr->label().text(F(">")).fontSize(+8).offset(1.5, -8).send();
  dwgsPtr->label().text(F("Output")).fontSize(+6).offset(0, 9).send();
  dwgsPtr->circle().filled().color(dwgsPtr->BLACK).radius(3).send();
  dwgsPtr->circle().filled().idx(z_idx + 2).color(dwgsPtr->WHITE).radius(2).send();
  dwgsPtr->popZero();

  if (has_PWM) {
    dwgsPtr->pushZero(20, 0, 0.5);
    dwgsPtr->label().text(F("&#126;")).fontSize(+8).offset(1.5, -8).send();
    dwgsPtr->label().text(F("PWM")).fontSize(+6).offset(0, 9).send();
    dwgsPtr->circle().filled().color(dwgsPtr->BLACK).radius(3).send();
    dwgsPtr->circle().filled().idx(z_idx + 3).color(dwgsPtr->WHITE).radius(2).send();
    dwgsPtr->popZero();
  }

  dwgsPtr->pushZero(-10, 0, 0.5);
  //		dwgsPtr->touchZone().cmd(buttonCmd_Input).size(2, 2).offset(-1, -1).filter(dwgsPtr->TOUCH).send();
  //		dwgsPtr->touchAction().cmd(buttonCmd_Input).action( // create the action
  //				dwgsPtr->circle().filled().idx(z_idx).color(dwgsPtr->BLACK).radius(4) //
  //				).send(); // send the touchAction
  //		dwgsPtr->touchAction().cmd(buttonCmd_Input).action( // create the action
  //				dwgsPtr->circle().filled().idx(z_idx + 1).color(dwgsPtr->WHITE).radius(2) //
  //				).send(); // send the touchAction
  //		dwgsPtr->touchAction().cmd(buttonCmd_Input).action( // create the action
  //				dwgsPtr->circle().filled().idx(z_idx + 2).color(dwgsPtr->WHITE).radius(2) //
  //				).send(); // send the touchAction
  dwgsPtr->out->print(F("|x~a~2~2~-1~-1"));
  dwgsPtr->out->print(F("|X~a~C`5~0~4~~"));
  dwgsPtr->out->print(F("|X~a~C`6~15~2~~"));
  dwgsPtr->out->print(F("|X~a~C`7~15~2~~"));

  if (has_PWM) {
    //			dwgsPtr->touchAction().cmd(buttonCmd_Input).action( // create the action
    //					dwgsPtr->circle().filled().idx(z_idx + 3).color(dwgsPtr->WHITE).radius(2) //
    //					).send(); // send the touchAction
    dwgsPtr->out->print(F("|X~a~C`8~15~2~~"));
  }

  dwgsPtr->popZero();

  dwgsPtr->pushZero(0, 0, 0.5);
  //		dwgsPtr->touchZone().cmd(buttonCmd_InputPullup).size(2, 2).offset(-1, -1).filter(dwgsPtr->TOUCH).send();
  //		dwgsPtr->touchAction().cmd(buttonCmd_InputPullup).action( // create the action
  //				dwgsPtr->circle().filled().idx(z_idx).color(dwgsPtr->WHITE).radius(2) //
  //				).send(); // send the touchAction
  //		dwgsPtr->touchAction().cmd(buttonCmd_InputPullup).action( // create the action
  //				dwgsPtr->circle().filled().idx(z_idx + 1).color(dwgsPtr->BLACK).radius(4) //
  //				).send(); // send the touchAction
  //		dwgsPtr->touchAction().cmd(buttonCmd_InputPullup).action( // create the action
  //				dwgsPtr->circle().filled().idx(z_idx + 2).color(dwgsPtr->WHITE).radius(2) //
  //				).send(); // send the touchAction
  dwgsPtr->out->print(F("|x~b~2~2~-1~-1"));
  dwgsPtr->out->print(F("|X~b~C`5~15~2~~"));
  dwgsPtr->out->print(F("|X~b~C`6~0~4~~"));
  dwgsPtr->out->print(F("|X~b~C`7~15~2~~"));

  if (has_PWM) {
    //			dwgsPtr->touchAction().cmd(buttonCmd_InputPullup).action( // create the action
    //					dwgsPtr->circle().filled().idx(z_idx + 3).color(dwgsPtr->WHITE).radius(2) //
    //					).send(); // send the touchAction
    dwgsPtr->out->print(F("|X~b~C`8~15~2~~"));
  }
  dwgsPtr->popZero();

  dwgsPtr->pushZero(+10, 0, 0.5);
  //		dwgsPtr->touchZone().cmd(buttonCmd_Output).size(2, 2).offset(-1, -1).filter(dwgsPtr->TOUCH).send();
  //		dwgsPtr->touchAction().cmd(buttonCmd_Output).action( // create the action
  //				dwgsPtr->circle().filled().idx(z_idx).color(dwgsPtr->WHITE).radius(2) //
  //				).send(); // send the touchAction
  //		dwgsPtr->touchAction().cmd(buttonCmd_Output).action( // create the action
  //				dwgsPtr->circle().filled().idx(z_idx + 1).color(dwgsPtr->WHITE).radius(2) //
  //				).send(); // send the touchAction
  //		dwgsPtr->touchAction().cmd(buttonCmd_Output).action( // create the action
  //				dwgsPtr->circle().filled().idx(z_idx + 2).color(dwgsPtr->BLACK).radius(4) //
  //				).send(); // send the touchAction
  dwgsPtr->out->print(F("|x~c~2~2~-1~-1"));
  dwgsPtr->out->print(F("|X~c~C`5~15~2~~"));
  dwgsPtr->out->print(F("|X~c~C`6~15~2~~"));
  dwgsPtr->out->print(F("|X~c~C`7~0~4~~"));
  if (has_PWM) {
    //			dwgsPtr->touchAction().cmd(buttonCmd_Output).action( // create the action
    //					dwgsPtr->circle().filled().idx(z_idx + 3).color(dwgsPtr->WHITE).radius(2) //
    //					).send(); // send the touchAction
    dwgsPtr->out->print(F("|X~c~C`8~15~2~~"));
  }
  dwgsPtr->popZero();

  if (has_PWM) {
    dwgsPtr->pushZero(+20, 0, 0.5);
    //			dwgsPtr->touchZone().cmd(buttonCmd_PWM).size(2, 2).offset(-1, -1).filter(dwgsPtr->TOUCH).send();
    //			dwgsPtr->touchAction().cmd(buttonCmd_PWM).action( // create the action
    //					dwgsPtr->circle().filled().idx(z_idx).color(dwgsPtr->WHITE).radius(2) //
    //					).send(); // send the touchAction
    //			dwgsPtr->touchAction().cmd(buttonCmd_PWM).action( // create the action
    //					dwgsPtr->circle().filled().idx(z_idx + 1).color(dwgsPtr->WHITE).radius(2) //
    //					).send(); // send the touchAction
    //			dwgsPtr->touchAction().cmd(buttonCmd_PWM).action( // create the action
    //					dwgsPtr->circle().filled().idx(z_idx + 2).color(dwgsPtr->WHITE).radius(2) //
    //					).send(); // send the touchAction
    //			dwgsPtr->touchAction().cmd(buttonCmd_PWM).action( // create the action
    //					dwgsPtr->circle().filled().idx(z_idx + 3).color(dwgsPtr->BLACK).radius(4) //
    //					).send(); // send the touchAction
    dwgsPtr->out->print(F("|x~p~2~2~-1~-1"));
    dwgsPtr->out->print(F("|X~p~C`5~15~2~~"));
    dwgsPtr->out->print(F("|X~p~C`6~15~2~~"));
    dwgsPtr->out->print(F("|X~p~C`7~15~2~~"));
    dwgsPtr->out->print(F("|X~p~C`8~0~4~~"));
    dwgsPtr->popZero();
  }

  fullUpdate(); // update with current state
  dwgsPtr->popZero(); // pop Base push and scale

}

void IO_Selector::fullUpdate() {

  dwgsPtr->circle().filled().idx(z_idx).color(dwgsPtr->WHITE).radius(2).send();//
  dwgsPtr->circle().filled().idx(z_idx + 1).color(dwgsPtr->WHITE).radius(2).send();//
  dwgsPtr->circle().filled().idx(z_idx + 2).color(dwgsPtr->WHITE).radius(2).send();//
  if (has_PWM) {
    dwgsPtr->circle().filled().idx(z_idx + 3).color(dwgsPtr->WHITE).radius(2).send();//
  }
  int idx = z_idx + value;
  dwgsPtr->circle().filled().idx(idx).color(dwgsPtr->BLACK).radius(2).send();//

}

boolean IO_Selector::haveStateChange() {
  return  (value != previousValue) || forceUpdate;
}

void IO_Selector::saveCurrentState() {
  previousValue = value;
  forceUpdate = false;
}

void IO_Selector::update() {
  if (haveStateChange()) {
    fullUpdate();
  }
  saveCurrentState();

}


