// Using Arduino/Genunio UNO
// Use Arduino V1.6.8 IDE
// Also works for Mega2560
// When run on Arduino Mega2560 the board name and the PWM pins will update Mega values
/*
   (c)2014-2017 Forward Computing and Control Pty. Ltd.
   NSW Australia, www.forward.com.au
   This code is not warranted to be fit for any purpose. You may only use it at your own risk.
   This generated code may be freely used for both private and commercial use
   provided this copyright is maintained.
*/

//
// download the pfodParser V3.1+ library from http://www.forward.com.au/pfod/pfodParserLibraries/index.html
#include <pfodParser.h>
#include "Uno_board.h"
#include "IO_Pin.h"
#include "IO_Selector.h"
#include "HL_Switch.h"
#include "ADC_5V_Display.h"
#include "Starter_Slider.h"
#include "Help_hidable.h"

int swap01(int); // method prototype for slider end swaps

pfodParser parser("V46"); // create a parser to handle the pfod messages

pfodDwgs dwgs(&parser);  // drawing support
// Commands for drawing controls only need to unique per drawing, you can re-use cmds 'a' in another drawing.

const int refreshTime = 2000; // 3sec when not editing
const int editRefreshTime = 0; // no refresh editing

IO_Selector ioSelector(false, &dwgs);
IO_Selector ioSelectorPWM(true, &dwgs);
Starter_Slider slider('P', &dwgs);
HL_Switch hlswitch('D', &dwgs);

// <<<<<<<<<<<<<<<<<< Note this code has been optimized so do not change the constants below as the code will stop working.
// The optimization uses the constants values instead of the variable name.
// see the dwgs.out->print statements further down
//
int selectedPinNo = -1; // -1 -> nothing selected

Help_hidable mainHelp('Z', &dwgs);
Help_hidable expandedHelp('Z', &dwgs); // different dwg so can reused cmd

const char expandCmd = 'x'; // used in dwg z only
int z_idx = 400;// optimization fix this z_idx  // + pfodDwgs::reserveIdx(1); // expand square index. Add 400 to avoid all the board dwg z_idx's

int closeTouchZone_order = 700;// optimization fix this z_idx + pfodDwgs::reserveIdx(4); // keep close on top
char closeCmd = 'X';

int loading_z_Idx = 800 + pfodDwgs::reserveIdx(3); // keep this on top of everything else

// dimensions are in 1/10"
const int colPadding = 5;
const int rowPadding = 4;
const float expandScaling = 3.0f;
int expandCol = colPadding + 9;
int expandRow = rowPadding + 5;
const int cols_uno = Uno_board::boardCols + 2 * colPadding;
const int rows_uno = Uno_board::boardRows + 2 * rowPadding - 3;
const int expandedRowSize = rows_uno / 2;

Uno_board uno_board(&dwgs);

IO_Pin d0Switch('a', &dwgs);
IO_Pin d1Switch('b', &dwgs);
IO_Pin d2Switch('c', &dwgs);
IO_Pin d3Switch('d', &dwgs);
IO_Pin d4Switch('e', &dwgs);
IO_Pin d5Switch('f', &dwgs);
IO_Pin d6Switch('g', &dwgs);
IO_Pin d7Switch('h', &dwgs);

IO_Pin d8Switch('i', &dwgs);
IO_Pin d9Switch('j', &dwgs);
IO_Pin d10Switch('k', &dwgs);
IO_Pin d11Switch('l', &dwgs);
IO_Pin d12Switch('m', &dwgs);
IO_Pin d13Switch('n', &dwgs);

ADC_5V_Display a0( &dwgs);
ADC_5V_Display a1( &dwgs);
ADC_5V_Display a2( &dwgs);
ADC_5V_Display a3( &dwgs);
ADC_5V_Display a4( &dwgs);
ADC_5V_Display a5( &dwgs);

const int switchArrayLen = 14;
const int analogArrayLen = 6;
IO_Pin* switchArray[switchArrayLen];
ADC_5V_Display *analogArray[analogArrayLen];

void clearSelections() {
  selectedPinNo = -1;
  for (int i = 0; i < switchArrayLen; i++) {
    // clear all selections
    switchArray[i]->clearSelected();
  }
}

// do this on connection
void forceFullUpdate() {
  clearSelections();
  for (int i = 0; i < switchArrayLen; i++) {
    switchArray[i]->forceFullUpdate();
  }
  for (int i = 0; i < analogArrayLen; i++) {
    analogArray[i]->skipUpdate();
  }
}

void onConnectForceUpdates() {
  forceFullUpdate();
}

// the setup routine runs once on reset:
void setup() {
  Serial.begin(9600);
  for (int i = 3; i > 0; i--) {
    // wait a few secs to see if we are being programmed
    delay(1000);
  }
  parser.connect(&Serial); // connect the parser to the i/o stream

  mainHelp.setBase(cols_uno - 1.5, 1.5);
  mainHelp.setPopUpOffset(-5, 7);

  mainHelp.setBaseScaling(0.75);
  mainHelp.setLabel(F("<-2>Drag red box\nto zoom in on the controls"));
  // 19 char max x 2 lines == ~ 38 x 12 height
  // round up to next even number
  mainHelp.setWidthHeight(30, 8);

  expandedHelp.setBase(cols_uno - 1.5, 1.5);
  expandedHelp.setPopUpOffset(-5, 7); // location of bottom right of popup

  expandedHelp.setBaseScaling(0.75);
  expandedHelp.setLabel(F("<-2>Click Digital pin to configure.\nAnalog pins are display only."));
  // 19 char max x 2 lines == ~ 38 x 12 height
  // round up to next even number
  expandedHelp.setWidthHeight(30, 8);

  slider.setLabel(F("<+3><bk>PWM "));
  slider.setBase(10, 8);
  hlswitch.setBase(35, 6);
  hlswitch.setBaseScaling(1.5);

  uno_board.setBase(colPadding, rowPadding - 2);



  d0Switch.setBase(Uno_board::boardCols, Uno_board::boardRows - 0);
  d1Switch.setBase(Uno_board::boardCols, Uno_board::boardRows - 1);
  d2Switch.setBase(Uno_board::boardCols, Uno_board::boardRows - 2);
  d3Switch.setBase(Uno_board::boardCols, Uno_board::boardRows - 3);
  d4Switch.setBase(Uno_board::boardCols, Uno_board::boardRows - 4);
  d5Switch.setBase(Uno_board::boardCols, Uno_board::boardRows - 5);
  d6Switch.setBase(Uno_board::boardCols, Uno_board::boardRows - 6);
  d7Switch.setBase(Uno_board::boardCols, Uno_board::boardRows - 7);


  d8Switch.setBase(Uno_board::boardCols, Uno_board::boardRows - 0 - 8);
  d9Switch.setBase(Uno_board::boardCols, Uno_board::boardRows - 1 - 8);
  d10Switch.setBase(Uno_board::boardCols, Uno_board::boardRows - 2 - 8);
  d11Switch.setBase(Uno_board::boardCols, Uno_board::boardRows - 3 - 8);
  d12Switch.setBase(Uno_board::boardCols, Uno_board::boardRows - 4 - 8);
  d13Switch.setBase(Uno_board::boardCols, Uno_board::boardRows - 5 - 8);

  int i = 0;
  switchArray[i++] = &d0Switch;
  switchArray[i++] = &d1Switch;
  switchArray[i++] = &d2Switch;
  switchArray[i++] = &d3Switch;
  switchArray[i++] = &d4Switch;
  switchArray[i++] = &d5Switch;
  switchArray[i++] = &d6Switch;
  switchArray[i++] = &d7Switch;
  switchArray[i++] = &d8Switch;
  switchArray[i++] = &d9Switch;
  switchArray[i++] = &d10Switch;
  switchArray[i++] = &d11Switch;
  switchArray[i++] = &d12Switch;
  switchArray[i++] = &d13Switch;


  a0.setLabel(F("A0"));
  a1.setLabel(F("A1"));
  a2.setLabel(F("A2"));
  a3.setLabel(F("A3"));
  a4.setLabel(F("A4"));
  a5.setLabel(F("A5"));

  a0.setBase(0, Uno_board::boardRows - 5);
  a1.setBase(0, Uno_board::boardRows - 4);
  a2.setBase(0, Uno_board::boardRows - 3);
  a3.setBase(0, Uno_board::boardRows - 2);
  a4.setBase(0, Uno_board::boardRows - 1);
  a5.setBase(0, Uno_board::boardRows - 0);

  i = 0;
  analogArray[i++] = &a0;
  analogArray[i++] = &a1;
  analogArray[i++] = &a2;
  analogArray[i++] = &a3;
  analogArray[i++] = &a4;
  analogArray[i++] = &a5;

}


// if cols in expaned view == normal view then
// box size == cols/expandScale,
void addExpandBox() {
  dwgs.rectangle().idx(z_idx).centered().color(dwgs.RED)
  .size(cols_uno / expandScaling, expandedRowSize / expandScaling).offset(expandCol, expandRow).send();
}

void disableExpandControl() {
  //  dwgs.touchZone().cmd(expandCmd).size(cols_uno, rows_uno)//
  //  .filter(dwgs.TOUCH_DISABLED).send();
  dwgs.out->print(F("|x~x~29~27~~`512"));
}

void addExpandControl() {
  addExpandBox();
  //  dwgs.touchZone().cmd(expandCmd).size(cols_uno, rows_uno)//
  //  .filter(dwgs.DOWN_UP).send();
  //  dwgs.touchAction().cmd(expandCmd).action( // create the action
  //    dwgs.rectangle().idx(z_idx).centered().color(dwgs.RED).size(cols_uno / expandScaling, expandedRowSize / expandScaling)
  //    .offset(dwgs.TOUCHED_COL, dwgs.TOUCHED_ROW)//
  //  ).send();
  dwgs.out->print(F("|x~x~29~27~~`256"));
  dwgs.out->print(F("|X~x~rc`400~9~9.7~4.3~c~r"));
}

int findSwitch(char dwgCmd) {
  for (int i = 0; i < switchArrayLen; i++) {
    if (switchArray[i]->getCmd() == dwgCmd) {
      return i;
    }
  }
  return -1; //not found
}

// digital pins 0 to 13
// analog pins A0 + 0 to 6
// for uno and mega skip pins 0,1 as they are used for coms
void readDigitalInputs() {
  for (byte i = 0; i < switchArrayLen; i++) {
    byte mode = switchArray[i]->getMode();
    if ((mode == IO_Pin::DIGITAL_INPUT) || (mode == IO_Pin::DIGITAL_INPUT_PULLUP)) {
      switchArray[i]->setValue(digitalRead(i));
    }
  }
  if (switchArray[13]->getValue() != 0) {
    uno_board.setLed(1);
  } else {
    uno_board.setLed(0);
  }
}

void readAnalogInputs() {
  for (byte i = 0; i < analogArrayLen; i++) {
    //  parser.print("reading analog:"); parser.println(A0 + i);
    analogArray[i]->setValue(analogRead(A0 + i));
  }
}

void setReadDigitalPin( byte pinNo) {
  if ((pinNo < 2) || (pinNo >= switchArrayLen)) {
    // skip 0,1
    return;
  } // else
  byte mode = switchArray[pinNo]->getMode();
  if (mode == IO_Pin::DIGITAL_INPUT) {
    pinMode(pinNo, INPUT);
    switchArray[pinNo]->setValue(digitalRead(pinNo));
  } else if (mode == IO_Pin::DIGITAL_INPUT_PULLUP) {
    pinMode(pinNo, INPUT_PULLUP);
    switchArray[pinNo]->setValue(digitalRead(pinNo));
  } else if (mode == IO_Pin::DIGITAL_OUTPUT) { // digital output
    pinMode(pinNo, OUTPUT);
    digitalWrite(pinNo, switchArray[pinNo]->getValue());
  } else { // PWM
    pinMode(pinNo, OUTPUT);
    analogWrite(pinNo, switchArray[pinNo]->getPwmValue());
  }
  if (pinNo == 13) { // set led state
    if (switchArray[13]->getValue() != 0) {
      uno_board.setLed(1);
    } else {
      uno_board.setLed(0);
    }
  }
}


// the loop routine runs over and over again forever:
void loop() {
  uint8_t cmd = parser.parse(); // parse incoming data from connection
  // parser returns non-zero when a pfod command is fully parsed
  if (cmd != 0) { // have parsed a complete msg { to }
    uint8_t* pfodFirstArg = parser.getFirstArg(); // may point to \0 if no arguments in this msg.
    pfod_MAYBE_UNUSED(pfodFirstArg); // may not be used, just suppress warning
    long pfodLongRtn; // used for parsing long return arguments, if any
    pfod_MAYBE_UNUSED(pfodLongRtn); // may not be used, just suppress warning
    if ('.' == cmd) {
      // pfodApp has connected and sent {.} , it is asking for the main menu
      // read digital input and analog values here for refresh
      readDigitalInputs();
      readAnalogInputs();
      clearSelections();
      if (!parser.isRefresh()) {
        sendMainMenu(); // send back the menu designed
      } else {
        sendMainMenuUpdate(); // menu is cached just send update
      }

      // now handle commands returned from button/vsliders
    } else if ('A' == cmd) { // user pressed menu item that loaded drawing with load cmd 'z'
      char dwgCmd = parser.parseDwgCmd(); // parse rest of dwgCmd, return first char of active cmd
      if (mainHelp.getCmd() == dwgCmd) { //
        dwgs.startUpdate();
        addExpandBox();
        dwgs.end();
      } else if (expandCmd == dwgCmd) { //
        int col = parser.getTouchedCol();
        int row = parser.getTouchedRow();
        expandCol = col;
        expandRow = row;
        showExpandedViewUpdate();
      } else {
        parser.print(F("{+}"));
      }
      // always send back a response or pfodApp will timeout

    } else if ('X' == cmd) { // user pressed expanded drawing loaded with load cmd 'x'
      char dwgCmd = parser.parseDwgCmd(); // parse rest of dwgCmd, return first char of active cmd
      // can be \0 if no dwg or no active areas defined
      if (expandedHelp.getCmd() == dwgCmd) { //
        dwgs.startUpdate(); // empty update to close help
        dwgs.end();
      } else if (closeCmd == dwgCmd) {
        closeExpandedView();
      } else {
        int pinNo = findSwitch(dwgCmd);
        if (pinNo >= 0) {
          if (selectedPinNo != pinNo) {
            if (selectedPinNo >= 0) {
              switchArray[selectedPinNo]->clearSelected();
            }
            if (pinNo < 2) {
              // skip Rx,Tx
              closeIOSelection();
            } else {
              selectedPinNo = pinNo;
              switchArray[selectedPinNo]->setSelected();
              ioSelector.setValue(switchArray[selectedPinNo]->getMode());
              ioSelectorPWM.setValue(switchArray[selectedPinNo]->getMode());
              ioSelector.forceFullUpdate();
              ioSelectorPWM.forceFullUpdate();
              slider.setValue(switchArray[selectedPinNo]->getPwmValue());
              hlswitch.setValue(switchArray[selectedPinNo]->getValue());
              hlswitch.forceFullUpdate();
              if (switchArray[selectedPinNo]->getMode() == IO_Pin::DIGITAL_PWM) {
                showIOSelectionPWM();
              } else if (switchArray[selectedPinNo]->getMode() == IO_Pin::DIGITAL_OUTPUT) {
                if (digitalPinHasPWM(selectedPinNo)) {
                  showIOSelectionOutputPWM();
                } else {
                  showIOSelectionOutput();
                }
              } else { // INPUT
                if (digitalPinHasPWM(selectedPinNo)) {
                  showIOSelectionInputPWM();
                } else {
                  showIOSelectionInput();
                }
              }
            }
          } else if (selectedPinNo == pinNo) {
            // pressed again
            // just close
            closeIOSelection();
          } else {
            // clear selection
            sendMainMenuUpdate();
          }
        } else {
          sendDrawingUpdates_x();
        }
      }

    } else if ('B' == cmd) { // user pressed expanded drawing loaded with load cmd 'b'
      char dwgCmd = parser.parseDwgCmd(); // parse rest of dwgCmd, return first char of active cmd
      if (closeCmd == dwgCmd) {
        closeIOSelection();
      } else {
        if (ioSelector.getCmd_Input() == dwgCmd) {
          ioSelector.setValue(IO_Pin::DIGITAL_INPUT);
        } else if (ioSelector.getCmd_InputPullup() == dwgCmd) {
          ioSelector.setValue(IO_Pin::DIGITAL_INPUT_PULLUP);
        } else if (ioSelector.getCmd_Output() == dwgCmd) {
          ioSelector.setValue(IO_Pin::DIGITAL_OUTPUT);
        } else if (ioSelector.getCmd_PWM() == dwgCmd) {
          ioSelector.setValue(IO_Pin::DIGITAL_PWM);
        }
        if (selectedPinNo >= 0) {
          switchArray[selectedPinNo]->setMode(ioSelector.getValue());
          setReadDigitalPin(selectedPinNo);
        }

        if (ioSelector.getValue() == IO_Pin::DIGITAL_PWM) {
          showIOSelectionPWM();
        } else if (ioSelector.getValue() == IO_Pin::DIGITAL_OUTPUT) {
          showIOSelectionOutput();
        } else {
          closeIOSelection();
        }
      }
      // always send back a response or pfodApp will timeout

    } else if ('P' == cmd) { // user pressed expanded drawing loaded with load cmd 'p'
      char dwgCmd = parser.parseDwgCmd(); // parse rest of dwgCmd, return first char of active cmd
      if (closeCmd == dwgCmd) {
        closeIOSelection();
      } else {
        if (ioSelectorPWM.getCmd_Input() == dwgCmd) { // user touched LED control
          ioSelectorPWM.setValue(IO_Pin::DIGITAL_INPUT);
        } else if (ioSelectorPWM.getCmd_InputPullup() == dwgCmd) { // user touched LED control
          ioSelectorPWM.setValue(IO_Pin::DIGITAL_INPUT_PULLUP);
        } else if (ioSelector.getCmd_Output() == dwgCmd) { // user touched LED control
          ioSelectorPWM.setValue(IO_Pin::DIGITAL_OUTPUT);
        } else if (ioSelectorPWM.getCmd_PWM() == dwgCmd) { // user touched LED control
          ioSelectorPWM.setValue(IO_Pin::DIGITAL_PWM);
        }
        if (selectedPinNo >= 0) {
          switchArray[selectedPinNo]->setMode(ioSelectorPWM.getValue());
          setReadDigitalPin(selectedPinNo);
        }

        if (ioSelectorPWM.getValue() == IO_Pin::DIGITAL_PWM) {
          showIOSelectionPWM();
        } else if (ioSelectorPWM.getValue() == IO_Pin::DIGITAL_OUTPUT) {
          showIOSelectionOutputPWM();
        } else {
          closeIOSelection();
        }
      }
      // always send back a response or pfodApp will timeout

    } else if ('C' == cmd) { // user pressed expanded drawing loaded with load cmd 'c'
      char dwgCmd = parser.parseDwgCmd(); // parse rest of dwgCmd, return first char of active cmd
      if (closeCmd == dwgCmd) {
        closeIOSelection(); // close both
        // skip update
      } else if (slider.getCmd() == dwgCmd) {
        slider.setValue(parser.getTouchedCol());
        switchArray[selectedPinNo]->setPwmValue(slider.getValue());
        setReadDigitalPin(selectedPinNo);
        refreshPWM();
      } else {
        parser.print(F("{+}")); // remove any help bubble
      }
    } else if ('D' == cmd) { // user pressed expanded drawing loaded with load cmd 'd'
      char dwgCmd = parser.parseDwgCmd(); // parse rest of dwgCmd, return first char of active cmd
      if (closeCmd == dwgCmd) {
        closeIOSelection(); // close both
      } else if (hlswitch.getCmd() == dwgCmd) {
        hlswitch.setValue((hlswitch.getValue() == 0) ? 1 : 0);
        switchArray[selectedPinNo]->setValue(hlswitch.getValue());
        setReadDigitalPin(selectedPinNo);
        refreshOutput();
      } else {
        parser.print(F("{+}")); // remove any help bubble
      }

    } else if ('b' == cmd) { // pfodApp is asking to load dwg 'b'
      if (!parser.isRefresh()) { // not refresh send whole dwg
        // get starting offset
        sendDrawing_b();
      } else { // refresh just update drawing state
        sendDrawingUpdates_b();
      }
    } else if ('p' == cmd) { // pfodApp is asking to load dwg 'p'
      if (!parser.isRefresh()) { // not refresh send whole dwg
        // get starting offset
        sendDrawing_p();
      } else { // refresh just update drawing state
        sendDrawingUpdates_p();
      }

    } else if ('c' == cmd) { // pfodApp is asking to load dwg 'c'
      if (!parser.isRefresh()) { // not refresh send whole dwg
        // get starting offset
        sendDrawing_c();
      } else { // refresh just update drawing state
        sendDrawingUpdates_c();
      }

    } else if ('d' == cmd) { // pfodApp is asking to load dwg 'd'
      if (!parser.isRefresh()) { // not refresh send whole dwg
        // get starting offset
        sendDrawing_d();
      } else { // refresh just update drawing state
        sendDrawingUpdates_d();
      }

    } else if ('a' == cmd) { // pfodApp is asking to load dwg 'a'
      // dwg 'a' is the main board layout and is loaded in 8 parts
      if (!parser.isRefresh()) { // not refresh send whole dwg
        // get starting offset
        parser.parseLong(pfodFirstArg, &pfodLongRtn); // get starting offset
        long startingOffset = pfodLongRtn;
        if (startingOffset == 0) {
          sendDrawing_a0();
        } else if (startingOffset == 1) {
          sendDrawing_a1();
        } else if (startingOffset == 2) {
          sendDrawing_a2();
        } else if (startingOffset == 3) {
          sendDrawing_a3();
        } else if (startingOffset == 4) {
          sendDrawing_a4();
        } else if (startingOffset == 5) {
          sendDrawing_a5();
        } else if (startingOffset == 6) {
          sendDrawing_a6();
        } else if (startingOffset == 7) {
          sendDrawing_a7();
        } else if (startingOffset == 8) {
          sendDrawing_a8();
        } else if (startingOffset == 9) {
          sendDrawing_a9();
        } else {
          parser.print(F("{}")); // finished
        }
      } else { // refresh just update drawing state
        sendDrawingUpdates_a();
      }

    } else if ('x' == cmd) { // pfodApp is asking to load dwg 'x'
      if (!parser.isRefresh()) { // not refresh send whole dwg
        sendDrawing_x();
      } else { // refresh just update drawing state
        sendDrawingUpdates_x();
      }

    } else if ('z' == cmd) { // pfodApp is asking to load dwg 'z'
      if (!parser.isRefresh()) { // not refresh send whole dwg
        sendDrawing_z();
      } else { // refresh just update drawing state
        sendDrawingUpdates_z();
      }

    } else if ('!' == cmd) {
      onConnectForceUpdates();
      // CloseConnection command
      closeConnection(parser.getPfodAppStream());
    } else {
      // unknown command
      parser.print(F("{}")); // always send back a pfod msg otherwise pfodApp will disconnect.
    }
  }
  //  <<<<<<<<<<<  Your other loop() code goes here

}

void closeConnection(Stream * io) {
  // add any special code here to force connection to be dropped
}

void addCloseButton(double c, double r, double scale) {
  dwgs.pushZero(c, r, scale);
  dwgs.circle().filled().idx(closeTouchZone_order).radius(1.8).offset(0, 0).color(dwgs.BLACK).send();
  dwgs.circle().filled().idx(closeTouchZone_order + 1).radius(1.5).offset(0, 0).color(dwgs.WHITE)
  .send();
  dwgs.label().idx(closeTouchZone_order + 2).text(F("x")).color(dwgs.BLACK).offset(0, -0.5).send();
  //  dwgs.touchZone().idx(closeTouchZone_order + 3).cmd(closeCmd).size(0.01, 0.01).send();
  //  dwgs.touchAction().cmd(closeCmd).action( //
  //    dwgs.circle().filled().idx(closeTouchZone_order + 1).radius(1.5).offset(0, 0).color(dwgs.BLACK))
  //  .send();
  dwgs.out->print(F("|x`703~X~0.01~0.01~~"));
  dwgs.out->print(F("|X~X~C`701~0~1.5~~"));

  dwgs.popZero();
}

void showExpandedViewUpdate() {
  clearSelections();
  parser.print(F("{;`")); // start an Update Menu pfod message
  parser.print(editRefreshTime);
  parser.print(F("|A"));
  parser.print(F("|X"));
  parser.print(F("|B-"));
  parser.print(F("|P-"));
  parser.print(F("|C-"));
  parser.print(F("|D-"));
  parser.print(F("}")); // close pfod message
}


void closeExpandedView() {
  clearSelections();
  parser.print(F("{;`")); // start an Update Menu pfod message
  parser.print(refreshTime);
  // send menu items
  parser.print(F("|X-"));
  parser.print(F("|A"));
  parser.print(F("|B-"));
  parser.print(F("|P-"));
  parser.print(F("|C-"));
  parser.print(F("|D-"));
  parser.print(F("}")); // close pfod message
}

void sendDrawing_d() {
  dwgs.start(50, 12); // background defaults to WHITE if omitted
  parser.sendVersion(); // send the parser version to cache this image
  dwgs.label().fontSize(+3).text(F("Output")).offset(15, 6).send();
  hlswitch.draw(); // draw the control
  dwgs.popZero();
  dwgs.end();
}

void sendDrawingUpdates_d() {
  dwgs.startUpdate();
  hlswitch.update(); // update with current state
  dwgs.end();
}

void sendDrawing_c() {
  dwgs.start(50, 13); // background defaults to WHITE if omitted
  parser.sendVersion(); // send the parser version to cache this image
  slider.draw(); // draw the control
  dwgs.popZero();
  dwgs.end();
}

void sendDrawingUpdates_c() {
  dwgs.startUpdate();
  slider.update(); // update with current state
  dwgs.end();
}

void sendDrawing_b() {
  dwgs.start(50, 15); // background defaults to WHITE if omitted
  parser.sendVersion(); // send the parser version to cache this image
  dwgs.pushZero(0, 1);
  ioSelector.draw(); // draw the control
  dwgs.popZero();
  addCloseButton(50 - 2, 15 - 2, 1);
  dwgs.end();
}

void sendDrawingUpdates_b() {
  dwgs.startUpdate();
  ioSelector.update(); // update with current state
  dwgs.end();
}

void sendDrawing_p() {
  dwgs.start(50, 15); // background defaults to WHITE if omitted
  parser.sendVersion(); // send the parser version to cache this image
  dwgs.pushZero(0, 1);
  ioSelectorPWM.draw(); // draw the control
  dwgs.popZero();
  addCloseButton(50 - 2, 15 - 2, 1);
  dwgs.end();
}

void sendDrawingUpdates_p() {
  dwgs.startUpdate();
  ioSelectorPWM.update(); // update with current state
  dwgs.end();
}

void showIOSelectionInput() {
  parser.print(F("{;`")); // start an Update Menu pfod message
  parser.print(editRefreshTime);
  // send menu items
  parser.print(F("|X"));
  parser.print(F("|B"));
  parser.print(F("|P-"));
  parser.print(F("|C-"));
  parser.print(F("|D-"));
  parser.print(F("}")); // close pfod message
}

void showIOSelectionInputPWM() {
  parser.print(F("{;`")); // start an Update Menu pfod message
  parser.print(editRefreshTime);
  // send menu items
  parser.print(F("|X"));
  parser.print(F("|B-"));
  parser.print(F("|P"));
  parser.print(F("|C-"));
  parser.print(F("|D-"));
  parser.print(F("}")); // close pfod message
}

void showIOSelectionPWM() {
  parser.print(F("{;`")); // start an Update Menu pfod message
  parser.print(editRefreshTime);
  // send menu items
  parser.print(F("|X"));
  parser.print(F("|P"));
  parser.print(F("|B-"));
  parser.print(F("|C"));
  parser.print(F("|D-"));
  parser.print(F("}")); // close pfod message
}

void showIOSelectionOutput() {
  parser.print(F("{;`")); // start an Update Menu pfod message
  parser.print(editRefreshTime);
  // send menu items
  parser.print(F("|X"));
  parser.print(F("|B"));
  parser.print(F("|P-"));
  parser.print(F("|C-"));
  parser.print(F("|D"));
  parser.print(F("}")); // close pfod message
}

void showIOSelectionOutputPWM() {
  parser.print(F("{;`")); // start an Update Menu pfod message
  parser.print(editRefreshTime);
  // send menu items
  parser.print(F("|X"));
  parser.print(F("|B-"));
  parser.print(F("|P"));
  parser.print(F("|C-"));
  parser.print(F("|D"));
  parser.print(F("}")); // close pfod message
}

void closeIOSelection() {
  clearSelections();
  parser.print(F("{;`")); // start an Update Menu pfod message
  parser.print(editRefreshTime);
  // send menu items
  parser.print(F("|X"));
  parser.print(F("|B-"));
  parser.print(F("|P-"));
  parser.print(F("|C-"));
  parser.print(F("|D-"));
  parser.print(F("}")); // close pfod message
}



void sendMainMenu() {
  clearSelections();
  // !! Remember to change the parser version string
  // every time you edit this method
  parser.print(F("{,")); // start a Menu screen pfod message
  // send menu background, format, prompt, refresh and version
  parser.print(F("~`"));
  parser.print(refreshTime);
  parser.sendVersion(); // send the menu version
  // send menu items
  parser.print(F("|+X-~x"));
  parser.print(F("|+B-~b"));
  parser.print(F("|+P-~p"));
  parser.print(F("|+C-~c"));
  parser.print(F("|+D-~d"));
  parser.print(F("|+A~z")); // change to embedded
  parser.print(F("}")); // close pfod message
}

void sendMainMenuUpdate() {
  clearSelections();
  parser.print(F("{;`")); // start an Update Menu pfod message
  parser.print(refreshTime);
  // send menu items
  parser.print(F("|A"));
  parser.print(F("|X-"));
  parser.print(F("|B-"));
  parser.print(F("|P-"));
  parser.print(F("|C-"));
  parser.print(F("|D-"));
  parser.print(F("}")); // close pfod message
}

void refreshOutput() {
  parser.print(F("{;`")); // start an Update Menu pfod message
  parser.print(editRefreshTime);
  // send menu items
  parser.print(F("|A")); // reloads z
  parser.print(F("|D")); // reloads pwm slider
  parser.print(F("}")); // close pfod message
}

void refreshPWM() {
  parser.print(F("{;`")); // start an Update Menu pfod message
  parser.print(editRefreshTime);
  // send menu items
  parser.print(F("|A")); // reloads z
  parser.print(F("|C")); // reloads pwm slider
  parser.print(F("}")); // close pfod message
}


void sendDrawing_x() {
  dwgs.start(cols_uno, expandedRowSize, dwgs.WHITE);
  parser.sendVersion();
  expandedHelp.draw();
  dwgs.pushZero(cols_uno / 2.0f, expandedRowSize / 2.0f, expandScaling - 0.1);
  dwgs.insertDwg().loadCmd('a').offset(expandCol, expandRow).send(); // offset in dwg being placed
  dwgs.popZero();
  addCloseButton(cols_uno - 1.15, expandedRowSize - 1.15, 0.58);
  dwgs.end();
}

void sendDrawingUpdates_x() {
  dwgs.startUpdate();
  dwgs.insertDwg().loadCmd('a').offset(expandCol, expandRow).send(); // offset in dwg being placed
  dwgs.end();
}

bool loadingShowing = false;
void sendDrawing_z() {
  dwgs.start(cols_uno, rows_uno, dwgs.WHITE);
  parser.sendVersion();
  mainHelp.draw();//
  mainHelp.hide();// hide it
  disableExpandControl();

  dwgs.insertDwg().loadCmd('a').offset(0, 0).send(); // offset in dwg being placed i.e. topleft
  dwgs.end();
}

void sendDrawingUpdates_z() {
  dwgs.startUpdate();
  dwgs.insertDwg().loadCmd('a').offset(0, 0).send(); // offset in dwg being placed i.e. topleft
  addExpandControl();
  mainHelp.unhide();
  dwgs.end();
}

void sendDrawing_a0() {
  // send outline of board
  dwgs.start(cols_uno, rows_uno, dwgs.WHITE, true);
  parser.sendVersion();
  draw_a0();
  dwgs.end();
}

void sendDrawing_a1() {
  dwgs.startUpdate(true);
  draw_a1();
  dwgs.end();
}

void sendDrawing_a2() {
  dwgs.startUpdate(true);
  draw_a2();
  dwgs.end();
}

void sendDrawing_a3() {
  dwgs.startUpdate(true);
  draw_a3();
  dwgs.end();
}

void sendDrawing_a4() {
  dwgs.startUpdate(true);
  draw_a4();
  dwgs.end();
}

void sendDrawing_a5() {
  dwgs.startUpdate(true);
  draw_a5();
  dwgs.end();
}

void sendDrawing_a6() {
  dwgs.startUpdate(true);
  draw_a6();
  dwgs.end();
}

void sendDrawing_a7() {
  dwgs.startUpdate(true);
  draw_a7();
  dwgs.end();
}

void sendDrawing_a8() {
  dwgs.startUpdate(true);
  draw_a8();
  dwgs.end();
}

void sendDrawing_a9() {
  dwgs.startUpdate(); // final part no more to come
  draw_a9();
  dwgs.end();
}

void draw_a0() {
  uno_board.drawFirst();//
  uno_board.drawOutline();
  loadingShowing = true;
  dwgs.label().idx(loading_z_Idx + 2).fontSize(-3).bold().color( dwgs.WHITE).text(F("Loading")).italic().offset(uno_board.boardCols / 2, 5).send();
  dwgs.label().idx(loading_z_Idx + 1).fontSize(-3).bold().color( dwgs.WHITE).text(F("Board Layout")).offset(uno_board.boardCols / 2, 7).italic().send();
  dwgs.pushZero(uno_board.boardCols / 2, 8); // avoids having to send offset for each updates to this label
  dwgs.label().idx(loading_z_Idx).fontSize(-3).bold().color( dwgs.WHITE).text(F(".........")).send();
  dwgs.popZero();
}

void draw_a1() {
  dwgs.label().idx(loading_z_Idx).fontSize(-3).bold().color( dwgs.WHITE).text(F(" ........")).send();
  uno_board.drawPins();
}

void draw_a2() {
  dwgs.label().idx(loading_z_Idx).fontSize(-3).bold().color( dwgs.WHITE).text(F("  .......")).send();
  drawD0toD3();
}

void draw_a3() {
  dwgs.label().idx(loading_z_Idx).fontSize(-3).bold().color( dwgs.WHITE).text(F("   ......")).send();
  drawD4toD7();
}

void draw_a4() {
  dwgs.label().idx(loading_z_Idx).fontSize(-3).bold().color( dwgs.WHITE).text(F("    .....")).send();
  dwgs.pushZero(0.0, -0.25);
  drawD8toD11();
  dwgs.popZero();
}

void draw_a5() {
  dwgs.label().idx(loading_z_Idx).fontSize(-3).bold().color( dwgs.WHITE).text(F("     ....")).send();
  dwgs.pushZero(0.0, -0.25);
  drawD12toD13();
  dwgs.popZero();
}

void draw_a6() {
  dwgs.label().idx(loading_z_Idx).fontSize(-3).bold().color( dwgs.WHITE).text(F("      ...")).send();
  dwgs.pushZero(0.0, 0);
  drawA0toA2();
  dwgs.popZero();
}

void draw_a7() {
  dwgs.label().idx(loading_z_Idx).fontSize(-3).bold().color( dwgs.WHITE).text(F("       ..")).send();
  dwgs.pushZero(0.0, 0);
  drawA3toA5();
  dwgs.popZero();
}

void draw_a8() {
  dwgs.label().idx(loading_z_Idx).fontSize(-3).bold().color(dwgs.WHITE).text(F("       .")).send();
  uno_board.update();// handles LED
  updateD0toD7();
}

void draw_a9() {
  drawingUpdates(); // do full update here to update rest of the pins
  uno_board.drawLast();//
}

void drawA0toA2() {
  a0.draw();
  a1.draw();
  a2.draw();
}

void drawA3toA5() {
  a3.draw();
  a4.draw();
  a5.draw();
}

void drawD0toD3() {
  d0Switch.draw();
  d1Switch.draw();
  d2Switch.draw();
  d3Switch.draw();
}

void drawD4toD7() {
  d4Switch.draw();
  d5Switch.draw();
  d6Switch.draw();
  d7Switch.draw();
}

void drawD8toD11() {
  d8Switch.draw();
  d9Switch.draw();
  d10Switch.draw();
  d11Switch.draw();
}

void drawD12toD13() {
  d12Switch.draw();
  d13Switch.draw();
}

void updateD0toD7() {
  d0Switch.update();
  d1Switch.update();
  d2Switch.update();
  d3Switch.update();
  d4Switch.update();
  d5Switch.update();
  d6Switch.update();
  d7Switch.update();
}

void updateD8toD13() {
  d8Switch.update();
  d9Switch.update();
  d10Switch.update();
  d11Switch.update();
  d12Switch.update();
  d13Switch.update();
}

void updateA0toA5() {
  a0.update();
  a1.update();
  a2.update();
  a3.update();
  a4.update();
  a5.update();
}

void drawingUpdates() {
  uno_board.update();// handles LED
  updateD0toD7();
  updateD8toD13();
  updateA0toA5();
}

void sendDrawingUpdates_a() {
  dwgs.startUpdate();
  if (loadingShowing) {
    dwgs.hide().idx(loading_z_Idx).send();
    dwgs.hide().idx(loading_z_Idx + 1).send();
    dwgs.hide().idx(loading_z_Idx + 2).send();
    loadingShowing = false;
  }
  drawingUpdates();
  dwgs.end();
}




int swap01(int in) {
  return (in == 0) ? 1 : 0;
}
// ============= end generated code =========

